#include <stdio.h>

/* int64 multiplication */
__int64 _allmul( __int64 a, __int64 b )
{
    __int64 r = (long) a * (long) b;
    
    int signa = (a & 0x8000000000000000) ? -1 : 1;
    int signb = (b & 0x8000000000000000) ? -1 : 1;
    int signr = (r & 0x8000000000000000) ? -1 : 1;
    
    if( (a & 0xffffffff00000000) || (b & 0xffffffff00000000) || (signa * signb != signr) )
    {
        printf( "WARNING: overflow in allmul()\n" );
        return (__int64) 1;
    }

    return r;
}


/* int64 shift left */
__int64 _allshl( __int64 a, __int64 b )
{
    __int64 r;
    long bl = (long) b;
    long *aw = (long*) &a;
    long *rw = (long*) &r;
    
    if( b <= 32 )
    {
        long ol;
        
        rw[0] = aw[0] << bl;
        rw[1] = aw[1] << bl;
        ol = aw[0] >> (32 - bl);
        rw[1] |= ol;
        return r;
    }

    if( b <= 64 )
    {
        rw[0] = 0;
        rw[1] = aw[0] << (bl - 32);
        return r;
    }

    printf( "WARNING: overflow in allshl(0x%08x%08x, 0x%08x )\n", aw[1], aw[0], bl );        
    return (long) a << (long) b;
}


/* int64 shift right */
__int64 _allshr( __int64 a, __int64 b )
{
    __int64 r;
    long bl = (long) b;
    long *aw = (long*) &a;
    long *bw = (long*) &b;
    long *rw = (long*) &r;
    
    if( b <= 32 )
    {
        long ol;
        
        rw[0] = aw[0] >> bl;
        rw[1] = aw[1] >> bl;
        ol = (aw[1] << (32 - bl)) >> (32 - bl);
        rw[1] |= ol;
        return r;
    }

    if( b <= 64 )
    {
        rw[0] = aw[1] >> (bl - 32);
        rw[1] = aw[1] >> 32;
        return r;
    }

    printf( "WARNING: overflow in allshr( 0x%08x%08x, 0x%08x%08x )\n", aw[1], aw[0], bw[1], bw[0] );
    return (long) a >> (long) b;
}


/* int64 division */
__int64 _alldiv( __int64 a, __int64 b )
{
    if( (a & 0xffffffff00000000) || (b & 0xffffffff00000000) )
        printf( "WARNING: overflow in alldiv()\n" );

    return (long) a / (long) b;
}


/* int64 remainder */
__int64 _allrem( __int64 a, __int64 b )
{
    if( (a & 0xffffffff00000000) || (b & 0xffffffff00000000) )
        printf( "WARNING: overflow in allrem()\n" );

    return (long) a % (long) b;
}
