/******************************************************************************
 *
 * Filename: java_lang_ClassLoader.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Native method implementations for java.lang.ClassLoader.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "../engine/config.h"

#include "java_lang_ClassLoader.h"

#include "../engine/util.h"
#include "../engine/globals.h"


/**
 * Initializes the Class loader.
 *
 * void init();
 */
void Java_java_lang_ClassLoader_init( JNIEnv* env, jobject self )
{
	/* ### doesn't need to do anything? */
}


/**
 * Get an InputStream on a given resource..  Will return null if no
 * resource with this name is found. <p>
 *
 * The resource name may be any system resource (e.g. follows CLASSPATH
 * order).
 *
 * @param	name	the name of the resource, to be used as is.
 * @return	an InputStream on the resource, or null if not found.
 * @since   JDK1.1
 *
 * static java.io.InputStream getSystemResourceAsStream0( java.lang.String name );
 */
jobject Java_java_lang_ClassLoader_getSystemResourceAsStream0( JNIEnv* env, jclass cls, jobject str )
{
	char filename[255];
	char* buf = filename;
	ObjRef* objref;
	ClassFile* cf;

	if( extractFromJavaLangString( filename, 255, str ) )
	{
#ifdef USES_ABSOLUTE_FILENAMES
		char absfilename[255];
		buf = absfilename;
		sprintf( absfilename, "%s%s%s", ABSOLUTE_FILENAME_ROOT, FILE_SEPARATOR, filename );
#endif /* USES_ABSOLUTE_FILENAMES */

		if( !(cf = getClass( env, "java/io/FileInputStream" )) )
			return NULL; /* getClass creates an exception for us */

		if( !(fdList[fdLast+1] = fopen( buf, "rb" )) )
		{
			if( verbose )
				printf( "WARNING: Failed to load system resource \"%s\"\n", filename );

			return NULL;
		}

		fdLast++;

		objref = newObjectRefAndObject( cf );
		addToList( objref, &objectPool );
		objref->data[0] = (u4) newJavaIoFileDescriptor( fdLast + 1 );
		addToList( (ObjRef*) objref->data[0], &objectPool );

		if( verbose )
			printf( "Opened system resource \"%s\"\n", filename );

		return objref;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return NULL;
}


/**
 * Finds the system class with the specified name, loading it in if 
 * necessary. 
 * <p>
 * A system class is a class loaded from the local file system in a 
 * platform- dependent way. It has no class loader. 
 *
 * @param      name   the name of the system <code>class</code>.
 * @return     a system class with the given name.
 * @exception  ClassNotFoundException  if it could not find a definition
 *               for the class.
 * @exception  NoClassDefFoundError    if the class is not found.
 * @since      JDK1.0
 *
 * java.lang.Class findSystemClass0( java.lang.String name );
 *     throws ClassNotFoundException;
 */
jclass Java_java_lang_ClassLoader_findSystemClass0( JNIEnv* env, jobject self, jobject str )
{
	char classname[255];

	if( extractFromJavaLangString( classname, 255, str ) )
	{
		ClassFile* cf;
		u4 n;

		for( n = 0; n < strlen( classname ); n++ )
		{
			if( classname[n] == '.' )
				classname[n] = '/';
		}

		if( !(cf = getClass( env, classname )) )
		{
			/* all exceptions get translated into ClassNotFoundExceptions */
			env->retcode = newCompleteException( env, ClassNotFoundException, classname );
			return NULL;
		}

		return cf->classobj;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return NULL;
}
