/******************************************************************************
 *
 * Filename: java_lang_Runtime.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  25th March 2002
 *
 * Purpose:  Native method implementations for java.lang.Runtime.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "../engine/config.h"
#include "../engine/util.h"
#include "../engine/globals.h"

#include "java_lang_Runtime.h"


/* private native String buildLibName(String pathname, String filename); */
jobject Java_java_lang_Runtime_buildLibName( JNIEnv* env, jobject self, jobject pathstr, jobject filestr )
{
	char filename[255];
	char pathname[255];
	ObjRef *str;

	if( extractFromJavaLangString( filename, 255, filestr )
		&& extractFromJavaLangString( pathname, 255, pathstr ) )
	{
		char buffer[512];

		sprintf( buffer, "%s%s%s.%s", pathname, FILE_SEPARATOR, filename, LIBRARY_EXTENSION );
		str = newJavaLangString( (u1*) buffer, strlen( buffer ) );
		return str;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return NULL;
}


/**
 * Helper for exit
 *
 * //__SYMC__
 * private native synchronized void exitInternal(int status);
 * //  private native void exitInternal(int status);
 * //__SYMC__
 */
void Java_java_lang_Runtime_exitInternal( JNIEnv* env, jobject self, jint retcode )
{
	printf( "WARNING: Runtime.exitInternal( %d ) called.\n", (int) retcode );
	env->retcode = 1; /* exit */
}


/**
 * Initializes the linker and returns the search path for shared libraries.
 *
 * private synchronized native String initializeLinkerInternal();
 */
jobject Java_java_lang_Runtime_initializeLinkerInternal( JNIEnv* env, jobject self )
{
	ObjRef* str = newJavaLangString( (u1*) ".", 1 );
	addToList( str, &objectPool );
	return str;
}


/**
 * Helper for load and loadLibrary
 *
 * Returns: -1 => OutOfMemory
 *           0 => UnsatisfiedLinkError
 *
 * private native int loadFileInternal(String filename);
 */
jint Java_java_lang_Runtime_loadFileInternal( JNIEnv* env, jobject self, jobject str )
{
	char filename[255];

	if( extractFromJavaLangString( filename, 255, str ) )
	{
		printf( "WARNING: Attempt to load library \"%s\"\n", filename );
		return 1;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return 0;
}


/**
 * Enables/Disables tracing of instructions.
 * If the <code>boolean</code> argument is <code>true</code>, this 
 * method asks the Java Virtual Machine to print out a detailed trace 
 * of each instruction in the Java Virtual Machine as it is executed. 
 * The virtual machine may ignore this request if it does not support 
 * this feature. The destination of the trace output is system 
 * dependent. 
 * <p>
 * If the <code>boolean</code> argument is <code>false</code>, this 
 * method causes the Java Virtual Machine to stop performing the 
 * detailed instruction trace it is performing. 
 *
 * @param   on   <code>true</code> to enable instruction tracing;
 *               <code>false</code> to disable this feature.
 * @since   JDK1.0
 *
 * public native void traceInstructions(boolean on);
 */
void Java_java_lang_Runtime_traceInstructions( JNIEnv* env, jobject self, jboolean on )
{
	instruction_debug = on;
}


/**
 * Enables/Disables tracing of method calls.
 * If the <code>boolean</code> argument is <code>true</code>, this 
 * method asks the Java Virtual Machine to print out a detailed trace 
 * of each method in the Java Virtual Machine as it is called. The 
 * virtual machine may ignore this request if it does not support 
 * this feature. The destination of the trace output is system dependent. 
 * <p>
 * If the <code>boolean</code> argument is <code>false</code>, this 
 * method causes the Java Virtual Machine to stop performing the 
 * detailed method trace it is performing. 
 *
 * @param   on   <code>true</code> to enable instruction tracing;
 *               <code>false</code> to disable this feature.
 * @since   JDK1.0
 *
 * public native void traceMethodCalls(boolean on);
 */
void Java_java_lang_Runtime_traceMethodCalls( JNIEnv* env, jobject self, jboolean on )
{
	method_debug = on;
}
