/******************************************************************************
 *
 * Filename: java_lang_Thread.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  25th March 2002
 *
 * Purpose:  Native method implementations for java.lang.Thread.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"
#include "../engine/engine.h"
#include "../engine/globals.h"
#include "../engine/util.h"

#include "java_lang_Thread.h"


/**
 * Returns a reference to the currently executing thread object.
 *
 * @return  the currently executing thread.
 * @since   JDK1.0
 *
 * public static native Thread currentThread();
 */
jobject Java_java_lang_Thread_currentThread( JNIEnv* env, jclass staticself )
{
	return threads[0];
}


/**
 * private native void setPriority0(int newPriority);
 */
void Java_java_lang_Thread_setPriority0( JNIEnv* env, jclass staticself, jint newPriority )
{
	printf( "WARNING: Thread.setPriority( %d ) dummied.\n", (int) newPriority );
}


/**
 * Causes this thread to begin execution; the Java Virtual Machine 
 * calls the <code>run</code> method of this thread. 
 * <p>
 * The result is that two threads are running concurrently: the 
 * current thread (which returns from the call to the 
 * <code>start</code> method) and the other thread (which executes its 
 * <code>run</code> method). 
 *
 * @exception  IllegalThreadStateException  if the thread was already
 *               started.
 * @see        java.lang.Thread#run()
 * @see        java.lang.Thread#stop()
 * @since      JDK1.0
 *
 * public synchronized native void start();
 */
void Java_java_lang_Thread_start( JNIEnv* env, jobject self )
{
	env->retcode = newCompleteException( env, InternalError, "Threads not supported." );
}
